<?php
/**
 * @file
 * Contains \Drupal\entity_pilot\Batch\AirTrafficController.
 */

namespace Drupal\entity_pilot\Batch;
use Drupal\entity_pilot\ArrivalInterface;
use Drupal\entity_pilot\Data\FlightManifestInterface;
use Drupal\entity_pilot\DepartureInterface;
use Drupal\entity_pilot\Exception\TransportException;
use Drupal\entity_pilot\Utility\FlightStub;
use Drupal\Core\Utility\Error;

/**
 * Does the bidding for the air traffic control service in a batch API.
 */
class AirTrafficController {

  /**
   * Batch API callback.
   *
   * @param \Drupal\entity_pilot\DepartureInterface $departure
   *   Entity to send.
   * @param array $context
   *   Context from batch API.
   *
   * @throws \Drupal\entity_pilot\Exception\TransportException
   *   When cannot be sent.
   */
  public static function takeoff(DepartureInterface $departure, &$context) {
    $air_traffic = \Drupal::service('entity_pilot.air_traffic_control');
    $translation = \Drupal::translation();
    try {
      $context['results'][] = $departure;
      $stub = FlightStub::create($departure->getRevisionId())
        ->setEntity($departure);
      $air_traffic->takeoff($stub);
    }
    catch (TransportException $e) {
      drupal_set_message($translation->translate('Departure %label could not be sent to Entity Pilot, please try again later or check logs for more detail.', [
        '%label' => $departure->label(),
      ]), 'error');
      drupal_set_message($e->getMessage() . '(' . $e->getCode() . ')', 'error');
      $message = '%type: !message in %function (line %line of %file).';
      $variables = Error::decodeException($e);
      \Drupal::logger('entity_pilot')->error($message, $variables);
    }
  }

  /**
   * Batch API finished callback.
   *
   * @param bool $success
   *   TRUE if successful
   * @param array $results
   *   Array of results.
   */
  public static function sent($success, $results) {
    $departure = reset($results);
    $translation = \Drupal::translation();
    if ($success) {
      /* @var \Drupal\entity_pilot\DepartureInterface $departure */
      $account = $departure->getAccount();
      $watchdog_args = array(
        '@type' => $departure->bundle(),
        '%info' => $departure->label(),
      );
      $t_args = array(
        '@type' => $account->label(),
        '%info' => $departure->label(),
      );
      \Drupal::logger('entity_pilot')->notice('@type: updated %info.', $watchdog_args);
      drupal_set_message($translation->translate('Departure for @type account named %info has been updated and sent.', $t_args));
    }
    else {
      drupal_set_message($translation->translate('Departure %label could not be sent to Entity Pilot, please try again later or check logs for more detail.', [
        '%label' => $departure->label(),
      ]), 'error');
    }
  }

  /**
   * Batch API callback to fetch contents for an arrival.
   *
   * @param \Drupal\entity_pilot\ArrivalInterface $arrival
   *   The arrival from which the contents are being fetched.
   * @param \Drupal\entity_pilot\Data\FlightManifestInterface $flight
   *   Incoming flight manifest.
   * @param string $site
   *   Site URI.
   * @param array $context
   *   Context of the batch operation.
   */
  public static function land(ArrivalInterface $arrival, FlightManifestInterface $flight, $site, &$context) {
    $arrival->setContents($flight->getTransposedContents($site, \Drupal::service('entity_pilot.transport'), $arrival->getAccount()))->save();
    $context['results']['arrival'] = $arrival;
  }

  /**
   * Batch API finished callback for fetching contents.
   */
  public static function landed() {
    drupal_set_message(\Drupal::translation()->translate('Fetched contents of remote flight - flight is now ready for approval.'));
  }

}
