<?php

/**
 * @file
 * Contains \Drupal\entity_pilot\Customs.
 */

namespace Drupal\entity_pilot;

use Drupal\Component\Graph\Graph;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Utility\Error;
use Drupal\entity_pilot\EntityResolver\UnsavedUuidResolverInterface;
use Drupal\rest\LinkManager\TypeLinkManagerInterface;
use Drupal\rest\Plugin\Type\ResourcePluginManager;
use GuzzleHttp\Exception\ClientException;
use Symfony\Component\Serializer\Serializer;

/**
 * Defines a class for dealing with filtering incoming entities.
 */
class Customs implements CustomsInterface {

  use LegacyMessagingTrait;
  use StringTranslationTrait;

  /**
   * The serializer service.
   *
   * @var \Symfony\Component\Serializer\Serializer
   */
  protected $serializer;

  /**
   * The rest resource plugin manager.
   *
   * @var \Drupal\rest\Plugin\Type\ResourcePluginManager
   */
  protected $resourceManager;

  /**
   * The typed link manager service.
   *
   * @var \Drupal\rest\LinkManager\TypeLinkManagerInterface
   */
  protected $typeLinkManager;

  /**
   * The entity manager.
   *
   * @var \Drupal\Core\Entity\EntityManagerInterface
   */
  protected $entityManager;

  /**
   * The graph of incoming objects.
   *
   * @var array
   */
  protected $graph = [];

  /**
   * A list of vertex objects keyed by their link.
   *
   * @var array
   */
  protected $vertexes = array();

  /**
   *  The plugin manager for determining if an entity already exists.
   *
   * @var \Drupal\entity_pilot\ExistsPluginManagerInterface
   */
  protected $existsPluginManager;

  /**
   * Array of deserialized entities.
   *
   * @var \Drupal\Core\Entity\EntityInterface[]
   */
  protected $denormalized = array();

  /**
   * Logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Resolver service.
   *
   * @var \Drupal\entity_pilot\EntityResolver\UnsavedUuidResolverInterface
   */
  protected $resolver;

  /**
   * Cache bin for screened entities.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * Constructs the default content manager.
   *
   * @param \Symfony\Component\Serializer\Serializer $serializer
   *   The serializer service.
   * @param \Drupal\rest\Plugin\Type\ResourcePluginManager $resource_manager
   *   The rest resource plugin manager.
   * @param \Drupal\Core\Entity\EntityManagerInterface $entity_manager
   *   The entity manager service.
   * @param \Drupal\rest\LinkManager\TypeLinkManagerInterface $type_link_manager
   *   The type link manager from the REST module.
   * @param \Drupal\entity_pilot\ExistsPluginManagerInterface $exists_manager
   *   The Entity Pilot exists plugin manager.
   * @param \Drupal\entity_pilot\EntityResolver\UnsavedUuidResolverInterface $resolver
   *     $resolver
   *   The entity resolver service.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $translation
   *   String translation service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   Logger factory service.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   Cache bin for screened entities.
   */
  public function __construct(Serializer $serializer, ResourcePluginManager $resource_manager, EntityManagerInterface $entity_manager, TypeLinkManagerInterface $type_link_manager, ExistsPluginManagerInterface $exists_manager, UnsavedUuidResolverInterface $resolver, TranslationInterface $translation, LoggerChannelFactoryInterface $logger_factory, CacheBackendInterface $cache) {
    $this->serializer = $serializer;
    $this->resourceManager = $resource_manager;
    $this->entityManager = $entity_manager;
    $this->typeLinkManager = $type_link_manager;
    $this->existsPluginManager = $exists_manager;
    $this->resolver = $resolver;
    $this->stringTranslation = $translation;
    $this->logger = $logger_factory->get('entity_pilot');
    $this->cache = $cache;
  }

  /**
   * {@inheritdoc}
   */
  public function screen(ArrivalInterface $flight, $display_errors = TRUE) {
    if ($cache = $this->cache->get($flight->id())) {
      $this->denormalized = $cache->data;
      return $cache->data;
    }
    $this->denormalized = [];
    foreach ($flight->getPassengers() as $id => $passenger) {
      $this->addEdges($passenger, $id);
    }
    $sorted = $this->sortTree($this->graph);
    $this->resetTree();
    foreach ($sorted as $id => $details) {
      $passenger = $flight->getPassengers($id);
      $type = $this->typeLinkManager->getTypeInternalIds($passenger['_links']['type']['href']);
      $class = $this->entityManager->getDefinition($type['entity_type'])->getClass();
      try {
        $unsaved_passenger = $this->serializer->denormalize($passenger, $class, 'hal_json');
        $this->resolver->add($unsaved_passenger);
        $this->denormalized[$id] = $unsaved_passenger;
      }
      catch (ClientException $e) {
        // Error occurred fetching a remote-file, it may no longer exist.
        $request = $e->getRequest();
        $url = $request->getUri();
        if ($display_errors) {
          $this->setMessage($this->t('An error occurred whilst fetching %url, this file will be ignored.', array(
            '%url' => $url,
          )), 'error');
        }
        $this->logger->error('An error occurred whilst fetching %url, this file will be ignored.', array(
          '%url' => $url,
        ));
        continue;
      }
    }
    $this->cache->set($flight->id(), $this->denormalized);
    return $this->denormalized;
  }

  /**
   * Adds a passenger to the dependency graph.
   *
   * @param array $passenger
   *   Normalized passenger in hal+json format.
   * @param string $id
   *   Passenger ID hash
   */
  protected function addEdges($passenger, $id) {
    // @todo - Do away with $id and just use self hrefs - requires changes to
    //   baggage handler.
    // Create a vertex for the graph.
    $vertex = $this->getVertex($id);
    $this->graph[$vertex->id]['edges'] = [];
    if (empty($passenger['_embedded'])) {
      // No dependencies to resolve.
      return;
    }
    // Here we need to resolve our dependencies;
    foreach ($passenger['_embedded'] as $embedded) {
      foreach ($embedded as $item) {
        $uuid = $item['uuid'][0]['value'];
        $edge = $this->getVertex($uuid);
        $this->graph[$vertex->id]['edges'][$edge->id] = TRUE;
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function previewPassenger($id) {
    if (!isset($this->denormalized[$id])) {
      throw new \InvalidArgumentException(sprintf('You must screen passengers first, passenger with ID %s has not been screened', $id));
    }
    return $this->denormalized[$id];
  }

  /**
   * Resets the dependency graph.
   */
  protected function resetTree() {
    $this->graph = [];
    $this->vertexes = array();
  }

  /**
   * Sorts current tree.
   *
   * @param array $graph
   *   Graph as an array.
   *
   * @return array
   *   Sorted dependency graph.
   */
  protected function sortTree($graph) {
    $graph_object = new Graph($graph);
    $sorted = $graph_object->searchAndSort();
    uasort($sorted, 'Drupal\Component\Utility\SortArray::sortByWeightElement');
    return array_reverse($sorted);
  }

  /**
   * Returns a vertex object for a given item link.
   *
   * Ensures that the same object is returned for the same item link.
   *
   * @param string $id
   *   Passenger ID
   *
   * @return object
   *   The vertex object.
   */
  protected function getVertex($id) {
    if (!isset($this->vertexes[$id])) {
      // @todo use a value object here.
      $this->vertexes[$id] = (object) array('id' => $id);
    }
    return $this->vertexes[$id];
  }

  /**
   * {@inheritdoc}
   */
  public function approve(ArrivalInterface $arrival) {
    $saved = array();
    $approved = $arrival->getApproved();
    foreach ($approved as $passenger_id) {
      if ($entity = $this->approvePassenger($passenger_id)) {
        $saved[] = $entity;
      }
    }
    return $saved;
  }

  /**
   * {@inheritdoc}
   */
  public function exists(EntityInterface $passenger) {
    return $this->existsPluginManager->exists($this->entityManager, $passenger);
  }

  /**
   * {@inheritdoc}
   */
  public function clearCache(FlightInterface $flight) {
    $this->denormalized = [];
    $this->cache->delete($flight->id());
  }

  /**
   * {@inheritdoc}
   */
  public function approvePassenger($passenger_id) {
    try {
      $entity = $this->previewPassenger($passenger_id);
      if ($exists = $this->exists($entity)) {
        $this->existsPluginManager->preApprove($entity, $exists);
      }
      $entity->save();
      return $entity;
    }
    catch (\Exception $e) {
      if (isset($entity)) {
        $this->logger->error('An error occurred whilst saving %label, this entity could not be updated/imported.', array(
          '%label' => $entity->label(),
        ));
      }
      $this->logger->error('%type: @message in %function (line %line of %file).', Error::decodeException($e));
    }
    return FALSE;
  }

}
