<?php

/**
 * @file
 * Contains \Drupal\entity_pilot\Data\FlightManifest.
 */

namespace Drupal\entity_pilot\Data;
use Drupal\Component\Serialization\Json;
use Drupal\entity_pilot\AccountInterface;
use Drupal\entity_pilot\Encryption\Encrypter;
use Drupal\entity_pilot\TransportInterface;

/**
 * Defines a value object for a flight object sent/received from EntityPilot.
 */
class FlightManifest implements FlightManifestInterface {

  /**
   * Remote ID.
   *
   * @var int
   */
  protected $id;

  /**
   * Flight identifier
   *
   * @var string
   */
  protected $info;

  /**
   * Array of normalized encrypted contents for the flight.
   *
   * @var array
   */
  protected $contents;

  /**
   * Account ID (carrier ID).
   *
   * @var int
   */
  protected $account;

  /**
   * Source site.
   *
   * @var string
   */
  protected $site;

  /**
   * Revision Log.
   *
   * @var string
   */
  protected $log;

  /**
   * Timestamp of change in UTC.
   *
   * @var int
   */
  protected $changed;

  /**
   * Black box key (password).
   *
   * @var string
   */
  protected $blackBoxKey;

  /**
   * Count of entities in flight.
   *
   * @var int
   */
  protected $count;

  /**
   * {@inheritdoc}
   */
  public static function fromArray($records, $decryption_secret) {
    $return = array();
    foreach ($records as $flight_details) {
      if (!isset($flight_details['id'])) {
        throw new \UnexpectedValueException('Cannot bulk create manifests for a flight without an ID.');
      }
      if (!empty($flight_details['contents'])) {
        $contents_payload = Json::decode($flight_details['contents']);
        $contents = [];
        foreach ($contents_payload as $uuid => $encrypted_entity) {
          try {
            $contents[$uuid] = Json::decode(Encrypter::decrypt($decryption_secret, base64_decode($encrypted_entity)));
          }
          catch (\InvalidCiphertextException $e) {
            watchdog_exception('entity_pilot', $e, sprintf('Could not decrypt entity %s', $uuid));
          }
        }
        $flight_details['contents'] = $contents;
      }
      $flight = static::create($flight_details);
      $return[$flight->getRemoteId()] = $flight;
    }
    return $return;
  }

  /**
   * {@inheritdoc}
   */
  public function getBlackBoxKey() {
    return $this->blackBoxKey;
  }

  /**
   * {@inheritdoc}
   */
  public static function create($values = array()) {
    $manifest = new static();
    foreach ($values as $key => $value) {
      $manifest->{$key} = $value;
    }
    return $manifest;
  }

  /**
   * {@inheritdoc}
   */
  public function setCarrierId($account_id) {
    $this->account = $account_id;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCarrierId() {
    return $this->account;
  }

  /**
   * {@inheritdoc}
   */
  public function setContents($contents) {
    $this->contents = $contents;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getContents($encoded = FALSE) {
    if (!$encoded) {
      return $this->contents;
    }
    return Json::encode($this->contents);
  }

  /**
   * {@inheritdoc}
   */
  public function setRemoteId($id) {
    $this->id = $id;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setBlackBoxKey($key) {
    $this->blackBoxKey = $key;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getRemoteId() {
    return $this->id;
  }

  /**
   * {@inheritdoc}
   */
  public function setInfo($info) {
    $this->info = $info;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    return $this->info;
  }

  /**
   * {@inheritdoc}
   */
  public function setLog($log) {
    $this->log = $log;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getLog() {
    return $this->log;
  }

  /**
   * {@inheritdoc}
   */
  public function setSite($site) {
    $this->site = $site;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getSite() {
    return $this->site;
  }

  /**
   * {@inheritdoc}
   */
  public function toArray($encryption_secret) {
    $date = new \DateTime('@' . $this->changed, new \DateTimeZone('UTC'));
    $contents = [];
    // Keep the UUID unencrypted.
    foreach ($this->contents as $uuid => $entity) {
      $contents[$uuid] = base64_encode(Encrypter::encrypt($encryption_secret, JSON::encode($entity)));
    }
    return array(
      'log' => $this->log,
      'contents' => JSON::encode($contents),
      'changed' => array(
        'date' => array(
          'year' => (int) $date->format('Y'),
          'month' => (int) $date->format('m'),
          'day' => (int) $date->format('d'),
        ),
        'time' => array(
          'hour' => (int) $date->format('H'),
          'minute' => (int) $date->format('i'),
        ),
      ),
      'account' => $this->account,
      'site' => $this->site,
      'info' => $this->info,
    );
  }

  /**
   * {@inheritdoc}
   */
  public function setChanged($changed) {
    $this->changed = $changed;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getChanged() {
    return $this->changed;
  }

  /**
   * {@inheritdoc}
   */
  public function getTransposedContents($new_site, TransportInterface $transport, AccountInterface $account) {
    if (isset($this->contents)) {
      $contents = $this->getContents(TRUE);
    }
    else {
      // We need to fetch contents from Entity Pilot.
      $contents = Json::encode($transport->getFlight($this->getRemoteId(), $account)->getContents());
    }
    return str_replace(str_replace('"', '', Json::encode($this->site)), str_replace('"', '', Json::encode($new_site)), $contents);
  }

  /**
   * {@inheritdoc}
   */
  public function getCount() {
    return $this->count;
  }

}
