<?php

/**
 * @file
 * Contains \Drupal\entity_pilot\Tests\ArrivalsTest.
 */

namespace Drupal\entity_pilot\Tests;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Queue\QueueInterface;
use Drupal\entity_pilot\ArrivalInterface;
use Drupal\entity_pilot\Data\FlightManifest;
use Drupal\entity_pilot\Entity\Arrival;
use Drupal\entity_pilot\MockTransportInterface;
use Drupal\node\Entity\Node;

/**
 * Ensures that Entity Pilot arrival functions work correctly.
 *
 * @group entity_pilot
 */
class ArrivalsTest extends EntityPilotTestBase {

  /**
   * We use the standard profile so we have the available fields.
   */
  protected $profile = 'standard';

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = array(
    'field_ui',
    'dynamic_entity_reference',
    'entity_pilot',
    'entity_test',
    'serialization',
    'hal',
    'entity_pilot_test',
    // @todo remove when https://www.drupal.org/node/2308745 lands
    'node',
  );

  /**
   * Permissions to grant admin user.
   *
   * @var array
   */
  protected $permissions = array(
    'administer entity_pilot accounts',
    'access administration pages',
    'administer entity_pilot arrivals',
    'view test entity',
    'edit any article content',
    'edit any page content',
  );

  /**
   * Tests creating an landing an arrival and importing on cron.
   */
  public function testArrivals() {
    $arrival = $this->doArrivalCreate();
    // Return to edit/approve.
    $this->drupalGet('admin/structure/entity-pilot/arrivals/' . $arrival->id() . '/approve');
    // Approval all bar the admin account.
    $edit = array(
      'approved_passengers[5f1af923-22f8-4799-9204-4f6f030bd879]' => 1,
      'approved_passengers[55ad425f-7832-4ff6-bb02-2f688bf95847]' => 1,
      'approved_passengers[cd215df5-242c-4844-a901-1dd566874727]' => 1,
      'approved_passengers[df0064ab-a7e1-4f30-8f06-6bb03032e052]' => 1,
      'approved_passengers[de511610-ae97-49a2-b65f-9548e54df2fa]' => 1,
      'approved_passengers[ea15274d-949c-4238-902d-45ca3c828ed1]' => 1,
      'approved_passengers[9887a6f6-23a8-4080-8231-49804054f681]' => 1,
      'approved_passengers[721b1351-a98f-4daa-842d-455c641fecbf]' => 1,
      'approved_passengers[19a8da05-5d5a-424d-8d00-e775744346ea]' => 1,
      'approved_passengers[51bad7d9-3994-4745-9220-32cb4f26c8e2]' => 1,
      'approved_passengers[01f1b727-d660-4647-8439-57be4e9cfce7]' => 1,
      'approved_passengers[7bec3ab2-cc87-488e-a607-7d70fb243e5f]' => 1,
      'approved_passengers[82c0651e-9bf9-4de7-9800-be1d6a5ae5a4]' => 1,
      'approved_passengers[ece8e4b2-737d-4818-af00-bea9078d2103]' => 1,
      'approved_passengers[ebac96dd-8b05-4abf-b301-1a1b3abe365c]' => 1,
      'approved_passengers[8798094b-9eaf-48e0-894a-74bb296d2f1f]' => 1,
      'approved_passengers[db099a7a-fc42-4765-8900-d233aa514a6b]' => 1,
      'approved_passengers[8384692b-379c-4067-b000-bea20ef3aaca]' => 1,
    );
    $this->drupalPostForm(NULL, $edit, t('Approve'));
    $this->assertText('Arrival for Primary account named Spring content refresh has been updated.');
    $this->assertLink(t('Queue'));
    $this->assertNoLink(t('Approve'));
    // Reload arrival.
    \Drupal::entityManager()
      ->getStorage('ep_arrival')
      ->resetCache(array($arrival->id()));
    /* @var \Drupal\entity_pilot\ArrivalInterface $arrival */
    $arrival = Arrival::load($arrival->id());
    $this->assertEqual($arrival->getRemoteId(), 1);
    $this->assertEqual(count($arrival->getApproved()), 18, 'Eighteen approved passengers on arrival');
    $this->drupalGet('admin/structure/entity-pilot/arrivals/' . $arrival->id() . '/queue');
    $this->drupalPostForm(NULL, array(), t('Queue'));
    // Reload arrival.
    \Drupal::entityManager()->getStorage('ep_arrival')->resetCache(array($arrival->id()));
    /* @var \Drupal\entity_pilot\ArrivalInterface $arrival */
    $arrival = Arrival::load($arrival->id());
    $this->assertTrue($arrival->isQueued());
    $this->assertEqual($arrival->getRemoteId(), 1);
    $this->assertNoLink(t('Approve'));
    $this->assertNoLink(t('Queue'));
    $this->assertLink(t('View'));

    /* @var QueueInterface $queue */
    $queue = \Drupal::service('queue')->get('entity_pilot_arrivals');
    $this->assertEqual($queue->numberOfItems(), 1);

    $this->cronRun();

    $this->assertEqual($queue->numberOfItems(), 0);

    $this->doArrivalTests($arrival);
  }

  /**
   * Tests creating an landing an arrival and importing via UI.
   */
  public function testArrivalLand() {
    $arrival = $this->doArrivalCreate();
    // Return to edit/approve.
    $this->drupalGet('admin/structure/entity-pilot/arrivals/' . $arrival->id() . '/approve');
    // Approval all bar the admin account.
    $edit = array(
      'approved_passengers[5f1af923-22f8-4799-9204-4f6f030bd879]' => 1,
      'approved_passengers[55ad425f-7832-4ff6-bb02-2f688bf95847]' => 1,
      'approved_passengers[cd215df5-242c-4844-a901-1dd566874727]' => 1,
      'approved_passengers[df0064ab-a7e1-4f30-8f06-6bb03032e052]' => 1,
      'approved_passengers[de511610-ae97-49a2-b65f-9548e54df2fa]' => 1,
      'approved_passengers[ea15274d-949c-4238-902d-45ca3c828ed1]' => 1,
      'approved_passengers[9887a6f6-23a8-4080-8231-49804054f681]' => 1,
      'approved_passengers[721b1351-a98f-4daa-842d-455c641fecbf]' => 1,
      'approved_passengers[19a8da05-5d5a-424d-8d00-e775744346ea]' => 1,
      'approved_passengers[51bad7d9-3994-4745-9220-32cb4f26c8e2]' => 1,
      'approved_passengers[01f1b727-d660-4647-8439-57be4e9cfce7]' => 1,
      'approved_passengers[7bec3ab2-cc87-488e-a607-7d70fb243e5f]' => 1,
      'approved_passengers[82c0651e-9bf9-4de7-9800-be1d6a5ae5a4]' => 1,
      'approved_passengers[ece8e4b2-737d-4818-af00-bea9078d2103]' => 1,
      'approved_passengers[ebac96dd-8b05-4abf-b301-1a1b3abe365c]' => 1,
      'approved_passengers[8798094b-9eaf-48e0-894a-74bb296d2f1f]' => 1,
      'approved_passengers[db099a7a-fc42-4765-8900-d233aa514a6b]' => 1,
      'approved_passengers[8384692b-379c-4067-b000-bea20ef3aaca]' => 1,
    );
    $this->drupalPostForm(NULL, $edit, t('Approve & Land'));
    $this->assertText('Arrival for Primary account named Spring content refresh has been updated.');
    // Reload arrival.
    \Drupal::entityManager()->getStorage('ep_arrival')->resetCache(array($arrival->id()));
    /* @var \Drupal\entity_pilot\ArrivalInterface $arrival */
    $arrival = Arrival::load($arrival->id());
    $this->assertTrue($arrival->isLanded());
    $this->assertEqual($arrival->getRemoteId(), 1);
    $this->assertNoLink(t('Approve'));
    $this->assertNoLink(t('Queue'));
    $this->assertLink(t('View'));

    $this->doArrivalTests($arrival);

    // Test two-way syncing.
    // Check that terms exist.
    $nodes = entity_load_multiple_by_properties('node', array(
      'uid' => 1,
      'title' => '12 Recipes with corn',
    ));
    $node = reset($nodes);
    $this->drupalPostForm('node/' . $node->id() . '/edit', array(
      'title[0][value]' => '12 recipes without corn',
      'field_image[0][alt]' => 'alt tag me sir',
    ), t('Save'));
    \Drupal::entityManager()->getStorage('node')->resetCache(array($node->id()));
    $updated = Node::load($node->id());
    $this->assertEqual($updated->label(), '12 recipes without corn');
    // Verify list shown.
    $this->drupalGet('admin/structure/entity-pilot/arrivals/add/primary');
    $edit = array(
      'remote_id' => 1,
      'info[0][value]' => 'Arrival two',
      'log' => 'The rain in spain falls mainly on the plain',
    );

    $this->drupalPostForm(NULL, $edit, t('Save'));
    $this->assertRaw(t('Arrival for @type account named %info has been created.', array(
      '@type' => 'Primary',
      '%info' => 'Spring content refresh',
    )));

    $arrivals = entity_load_multiple_by_properties('ep_arrival', array(
      'remote_id' => 1,
    ));
    $arrival = end($arrivals);
    $this->assertUrl('admin/structure/entity-pilot/arrivals/' . $arrival->id() . '/approve');
    // Approval only the corn story bar the admin account.
    $edit = array(
      'approved_passengers[8384692b-379c-4067-b000-bea20ef3aaca]' => 1,
    );
    $this->drupalPostForm(NULL, $edit, t('Approve & Land'));
    $this->assertText('Arrival for Primary account named Spring content refresh has been updated.');
    // Check the name updated again.
    \Drupal::entityManager()->getStorage('node')->resetCache(array($node->id()));
    $updated = Node::load($node->id());
    $this->assertEqual($updated->label(), '12 Recipes with corn');
  }

  /**
   * Sets up an arrival for testing.
   *
   * @return \Drupal\entity_pilot\ArrivalInterface
   *   The created arrival.
   */
  protected function doArrivalCreate() {
    $this->drupalLogin($this->adminUser);
    // Visit admin structure.
    $this->drupalGet('admin/structure');
    $this->clickLink('Entity Pilot');
    $this->clickLink('Entity Pilot Arrivals');
    $this->assertText('There is no Entity Pilot Arrival yet');

    // Add new arrival without an account.
    $this->clickLink('New arrival');

    $this->assertText(t('No Entity Pilot accounts have been created.'));

    // Create an account.
    $this->createAccount('Primary', NULL, NULL, 'Magic ponies fly these skies.');

    // Simulate service outage.
    /* @var MockTransportInterface $transport */
    $transport = \Drupal::service('entity_pilot.transport');
    $transport->setExceptionReturn("stuff wen't bad");

    // Reload.
    $this->drupalGet('admin/structure/entity-pilot/arrivals/add');
    $this->assertRaw(t('An error occurred connecting to the Entity Pilot backend, please try again later or visit the <a href="https://entitypilot.com/status">Entity Pilot status page</a>.'));
    $flights = Json::decode(file_get_contents(__DIR__ . '/Data/flights.json'));
    $transport->setQueryReturn(FlightManifest::fromArray($flights, hex2bin("a22a0b2884fd73c4e211d68e1f031051")));
    // Verify form is not shown.
    $this->assertNoField('info[0][value]', 'Did not find description field');

    // Create another account.
    $this->createAccount('Secondary', NULL, NULL, 'The airline of choice for top-shelf goats.');

    // Verify list shown.
    $this->drupalGet('admin/structure/entity-pilot/arrivals/add');
    // Verify form is not shown.
    $this->assertNoField(('info[0][value]'), 'Did not find description field');
    // Verify descriptions shown.
    $this->assertText('The airline of choice for top-shelf goats.');
    $this->assertText('Magic ponies fly these skies.');
    $this->assertLink('Secondary');

    // Use the primary account.
    $this->clickLink('Primary');

    $this->assertRaw(t('New Entity Pilot arrival'));

    $this->assertFieldById('edit-remote-id-1');
    $this->assertFieldById('edit-remote-id-2');
    $this->assertFieldById('edit-search');
    $this->assertFieldById('edit-go');

    $edit = array(
      'remote_id' => 1,
      'info[0][value]' => 'Arrival one',
      'log' => 'The rain in spain falls mainly on the plain',
    );

    $this->drupalPostForm(NULL, $edit, t('Save'));
    $this->assertRaw(t('Arrival for @type account named %info has been created.', array(
      '@type' => 'Primary',
      '%info' => 'Spring content refresh',
    )));

    $this->assertText('Spring content refresh');

    $arrivals = entity_load_multiple_by_properties('ep_arrival', array(
      'remote_id' => 1,
    ));
    $this->assertEqual(1, count($arrivals), 'Arrival was saved');
    /* @var \Drupal\entity_pilot\ArrivalInterface $arrival */
    $arrival = reset($arrivals);
    $this->assertUrl('admin/structure/entity-pilot/arrivals/' . $arrival->id() . '/approve');
    $this->assertEqual($arrival->getRemoteId(), 1);
    $this->assertTitle(t('Approve Spring content refresh') . ' | Drupal');
    $this->assertEqual(count($arrival->getPassengers()), 19, '19 passengers on arrival');
    $this->assertText('Vegan');
    $this->assertText('admin');
    $this->assertText('Hidden health benefits of hazelnuts');
    $this->assertText('10 Things to pack for the best holiday');
    $this->assertText('Geography');
    $this->assertLink(t('Preview'));
    $this->assertLink(t('Diff'));

    // View user diff.
    $this->clickLink(t('Diff'));
    $this->assertTitle('View changes | Drupal');
    $this->assertText('Incoming');
    $this->assertText('Existing');
    $this->assertText('505718d0-d918-4474-ba02-2c4108b4c7aa');

    // Return to edit/approve.
    $this->drupalGet('admin/structure/entity-pilot/arrivals/' . $arrival->id() . '/approve');
    // Check the preview.
    $this->drupalGet('admin/structure/entity-pilot/arrivals/' . $arrival->id() . '/approve/preview/de511610-ae97-49a2-b65f-9548e54df2fa');
    $this->assertText('Pickled Schlitz fixie, butcher forage');
    $this->assertText('Submitted by admin');
    $this->assertText('Vegan');
    $this->assertText('Healthy Eating');
    $image = $this->cssSelect('.node .field--name-field-image img');
    $this->assertTrue(preg_match('/hazelnuts-small/', $image[0]['src']));
    return $arrival;
  }

  /**
   * Tests the arrival.
   *
   * @param \Drupal\entity_pilot\ArrivalInterface $arrival
   *   The arrival to test
   */
  protected function doArrivalTests(ArrivalInterface $arrival) {
    \Drupal::entityManager()
      ->getStorage('ep_arrival')
      ->resetCache(array($arrival->id()));
    /* @var \Drupal\entity_pilot\ArrivalInterface $arrival */
    $arrival = Arrival::load($arrival->id());
    $this->assertEqual($arrival->getRemoteId(), 1);
    $this->assertTrue($arrival->isLanded());

    // Check that terms exist.
    $tags = entity_load_multiple_by_properties('taxonomy_term', array(
      'vid' => 'tags',
    ));
    $this->assertEqual(count($tags), 11);
    $term = reset($tags);
    $this->assertEqual($term->label(), 'Vegan');
    $term2 = end($tags);
    $this->assertEqual($term2->label(), 'Recipes');

    // Check that terms exist.
    $nodes = entity_load_multiple_by_properties('node', array(
      'uid' => 1,
    ));
    $titles = [
      'About Us',
      '12 Recipes with corn',
      'Hidden health benefits of hazelnuts',
      '10 Things to pack for the best holiday',
    ];
    /* @var \Drupal\node\NodeInterface $node */
    foreach ($nodes as $id => $node) {
      $label = $node->label();
      unset($titles[array_search($label, $titles)]);
      $this->pass("Found node titled $label");
      if ($label !== 'About Us') {
        // Assert file was found.
        $this->assertTrue($node->field_image->entity->id());
      }
    }
  }

}
