<?php

/**
 * @file
 * Contains \Drupal\Tests\entity_pilot\Unit\BaggageHandlerTest.
 */

namespace Drupal\Tests\entity_pilot\Unit;

use Drupal\Core\Cache\Cache;
use Drupal\entity_pilot\BaggageHandler;
use Drupal\Tests\UnitTestCase;

/**
 * @coversDefaultClass \Drupal\entity_pilot\BaggageHandler
 * @group entity_pilot
 */
class BaggageHandlerTest extends UnitTestCase {

  /**
   * @var \Drupal\entity_pilot\BaggageHandler
   */
  protected $baggageHandler;

  /**
   * The serializer service.
   *
   * @var \Symfony\Component\Serializer\Serializer|\PHPUnit_Framework_MockObject_MockObject
   */
  protected $serializer;

  /**
   * The entity manager.
   *
   * @var \Drupal\Core\Entity\EntityManagerInterface|\PHPUnit_Framework_MockObject_MockObject
   */
  protected $entityManager;

  /**
   * Cache backend
   *
   * @var \Drupal\Core\Cache\MemoryBackend|\PHPUnit_Framework_MockObject_MockObject
   */
  protected $cache;

  /**
   * Mock entity
   *
   * @var \Drupal\node\NodeInterface|\PHPUnit_Framework_MockObject_MockObject
   */
  protected $entity;

  /**
   * Sets up the test.
   */
  protected function setUp() {
    $this->serializer = $this->getMock('\Symfony\Component\Serializer\Serializer');
    $this->entityManager = $this->getMock('\Drupal\Core\Entity\EntityManagerInterface');
    $this->entity = $this->getMock('\Drupal\node\NodeInterface');
    $this->entity->expects($this->any())
      ->method('getEntityTypeId')
      ->willReturn('node');
    $this->entity->expects($this->any())
      ->method('id')
      ->willReturn(1);
    $this->cache = $this->getMock('Drupal\Core\Cache\CacheBackendInterface');
    $this->entity->expects($this->any())
      ->method('uuid')
      ->willReturn('entity-uuid');
    $this->baggageHandler = new BaggageHandler($this->serializer, $this->entityManager, $this->cache);
  }

  /**
   * Tests reset().
   *
   * @covers ::reset
   * @covers ::__construct
   */
  public function testReset() {
    $cid = sprintf('ep__%s__%s', $this->entity->getEntityTypeId(), $this->entity->id());
    $this->cache->expects($this->exactly(2))
      ->method('get')
      ->with($cid)
      ->willReturn((object) ['data' => 'magic ponies']);
    // Cold static cache should trigger cache get call.
    $dependencies = $this->baggageHandler->calculateDependencies($this->entity);
    $this->assertEquals($dependencies, 'magic ponies');
    // Warmed static cache.
    $dependencies = $this->baggageHandler->calculateDependencies($this->entity);
    $this->assertEquals($dependencies, 'magic ponies');
    $this->baggageHandler->reset();
    // Cold static cache should trigger cache get call again.
    $dependencies = $this->baggageHandler->calculateDependencies($this->entity);
    $this->assertEquals($dependencies, 'magic ponies');
  }

  /**
   * Tests calculateDependencies().
   *
   * @covers ::calculateDependencies
   */
  public function testCalculateDependencies() {
    $dependant1 = $this->getMock('\Drupal\node\NodeInterface');
    $dependant1->expects($this->any())
      ->method('getEntityTypeId')
      ->willReturn('node');
    $dependant1->expects($this->any())
      ->method('id')
      ->willReturn(2);
    $dependant1->expects($this->any())
      ->method('uuid')
      ->willReturn('pony-uuid');
    $dependant2 = $this->getMock('\Drupal\node\NodeInterface');
    $dependant2->expects($this->any())
      ->method('getEntityTypeId')
      ->willReturn('node');
    $dependant2->expects($this->any())
      ->method('id')
      ->willReturn(3);
    $dependant2->expects($this->any())
      ->method('uuid')
      ->willReturn('fooey-uuid');
    $this->entity->expects($this->any())
      ->method('referencedEntities')
      ->willReturn([$dependant1, $dependant2]);
    $dependant1->expects($this->any())
      ->method('referencedEntities')
      ->willReturn([]);
    $dependant2->expects($this->any())
      ->method('referencedEntities')
      ->willReturn([]);
    $this->cache->expects($this->any())
      ->method('get')
      ->willReturn(FALSE);
    $this->cache->expects($this->any())
      ->method('set')
      ->willReturnCallback(function($cid, $data, $ttl, $tags) use ($dependant1, $dependant2) {
        $expected = [
          'ep__node__2' => ['ep__node__2', [], Cache::PERMANENT, ['ep__node__2']],
          'ep__node__3' => ['ep__node__3', [], Cache::PERMANENT, ['ep__node__3']],
          'ep__node__1' => ['ep__node__1', [
            'pony-uuid' => $dependant1,
            'fooey-uuid' => $dependant2,
          ], Cache::PERMANENT, [
            'ep__node__1',
            'ep__node__2',
            'ep__node__3',
          ]],
        ];
        $this->assertEquals($expected[$cid], func_get_args());
      });
    $dependencies = $this->baggageHandler->calculateDependencies($this->entity);
    $this->assertEquals($dependencies, [
      'pony-uuid' => $dependant1,
      'fooey-uuid' => $dependant2,
    ]);
    // Second call should use static.
    $this->baggageHandler->calculateDependencies($this->entity);
    // Clear the static cache.
    $this->baggageHandler->reset();
    $return = (object) ['data' => ['giant asthmatic pandas']];
    // Prime the cache.
    $this->cache->expects($this->any())
      ->method('get')
      ->willReturnCallback(function ($cid) use ($dependant1, $dependant2) {
        if ($cid == 'ep__node__1') {
          return [
            'pony-uuid' => $dependant1,
            'fooey-uuid' => $dependant2,
          ];
        }
        return FALSE;
      });
    $dependencies = $this->baggageHandler->calculateDependencies($this->entity);
    $this->assertEquals($dependencies, [
      'pony-uuid' => $dependant1,
      'fooey-uuid' => $dependant2,
    ]);
  }

  /**
   * Tests calculateDependencies() with a circular reference.
   *
   * @covers ::calculateDependencies
   */
  public function testCalculateCircularDependencies() {
    $dependant1 = $this->getMock('\Drupal\node\NodeInterface');
    $dependant1->expects($this->any())
      ->method('getEntityTypeId')
      ->willReturn('node');
    $dependant1->expects($this->any())
      ->method('id')
      ->willReturn(2);
    $dependant1->expects($this->any())
      ->method('uuid')
      ->willReturn('pony-uuid');
    $dependant2 = $this->getMock('\Drupal\node\NodeInterface');
    $dependant2->expects($this->any())
      ->method('getEntityTypeId')
      ->willReturn('node');
    $dependant2->expects($this->any())
      ->method('id')
      ->willReturn(3);
    $dependant2->expects($this->any())
      ->method('uuid')
      ->willReturn('fooey-uuid');
    $this->entity->expects($this->any())
      ->method('referencedEntities')
      ->willReturn([$dependant1]);
    $dependant1->expects($this->any())
      ->method('referencedEntities')
      ->willReturn([$this->entity, $dependant2]);
    $dependant2->expects($this->any())
      ->method('referencedEntities')
      ->willReturn([$dependant1]);
    $this->cache->expects($this->any())
      ->method('set')
      ->willReturn(TRUE);
    $this->cache->expects($this->any())
      ->method('get')
      ->willReturn(FALSE);
    $dependencies = $this->baggageHandler->calculateDependencies($this->entity);
    $this->assertEquals([
      'pony-uuid' => $dependant1,
      'fooey-uuid' => $dependant2,
    ], $dependencies);
  }

}
